/* -*- C -*- */
/* Time-stamp: "2006-01-16 10:01:04 fujiwara" */

/**********************************************************************/
/*  Copyright (C) 2000, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: atoef.c,v 1.1.1.1 2002/04/07 07:18:09 fujiwara Exp $";

#include "exfloat.h"

static void exflt_parse(const char str[], char dest[], exflib_Qword *sign, long *e)
{
    long _e, e_sign;

    /* skip leading white space */
    while((*str == ' ') || (*str == '\t'))
	str++;

    /* sign */
    *sign = 0;
    if(*str == '-'){
	*sign = EXFLIB_SIGN;
	str++;
    }else if(*str == '+'){
	str++;
    }

    /* skip white space and leading zeros */
    while(*str == ' ' || *str == '\t')
	str++;

    /* integer part */
    *e = -1;

    while('0' <= *str && *str <= '9'){
	*dest++ = *str++;
	*e += 1;
    }

    /* decimal point */
    if(*str == '.'){
	str++;
	while(('0' <= *str) && (*str <= '9'))
	    *dest++ = *str++;
    }

    *dest = '\0';

    /* exponent part */
    if((*str == 'e') || (*str == 'E')){
	str++;

	if(*str == '-'){
	    e_sign = -1;
	    str++;
	} else if(*str == '+'){
	    e_sign = 1;
	    str++;
	} else {
	    e_sign = 1;
	}

	for(_e = 0; *str; str++){
	    _e *= 10;
	    _e += *str - '0';
	}

	if(e_sign > 0)
	    *e += _e;
	else
	    *e -= _e;
    }
    return;
}	

static void exflt_conv_radix(char deci[], exflib_uQword u[], long usize)
{
    exflib_uQword u_0, base;
    exflib_uQword unit[EXFLIB_CONST_MAX_PR], tmp[EXFLIB_CONST_MAX_PR];
    long top;

    long i;

    u[0] = *deci++ - '0';
    unit[0] = 1;

    for(i = 1; i < usize; i++)
	u[i] = unit[i] = 0;

    top = 0;

    while(*deci){

	u_0 = 0;
	for(base = 1; (base <= 0x10000) && (*deci); base *= 10, deci++){
	    u_0 *= 10;
	    u_0 += *deci - '0';
	}

	if(base > 1)
	    exflib_array_idiv(unit, unit, (exflib_uQword)base, usize, &top);
	if(top >= usize)
	    break;
	exflib_array_imul(tmp, unit, u_0, usize);
	exflib_array_add(u, u, tmp, usize);
    }
    
    return;
}

static void exflib_convert_pow10(long d, exflib_Qword *e_int, exflib_uQword u[], long usize)
{
    static exflib_uQword log_2_10[] = {
#include "log_2_10.fixed"
    };
    static exflib_uQword ln2[] = {
#include "ln2.fixed"
    };

    exflib_uQword e_frac[EXFLIB_CONST_MAX_PR], term[EXFLIB_CONST_MAX_PR], y[EXFLIB_CONST_MAX_PR];
    long i, top;

    if(d >= 0){

	exflib_array_imul(e_frac, log_2_10, d, usize);
	*e_int = e_frac[0];

    } else {

	exflib_array_imul(e_frac, log_2_10, -d, usize);
	*e_int = -e_frac[0] - 1;

	/* bit reverse and add 1ulp */
	for(i = 0; i < usize; i++)
	    e_frac[i] = ~e_frac[i];
	for(i = usize - 1; e_frac[i] == 0xffffffffffffffffULL; i--)
	    e_frac[i] = 0;
	e_frac[i]++;
    }
    e_frac[0] = 0;

    /* calculate 2^(e_frac) */

    exflib_array_mul(y, ln2, e_frac, usize);
    for(i = 0; i < usize; i++)
	u[i] = term[i] = y[i];
    u[0]++;

    /* Taylor expansion */
    for(top = 0, i = 2; top < usize; i++){
	exflib_array_mul(term, term, y, usize);
	if((top > 0) && (term[top - 1] != 0))
	    top--;
	exflib_array_idiv(term, term, (exflib_uQword)i, usize, &top);
	exflib_array_add(u, u, term, usize);
    }

    return;
}

void exflib_float_atoef(const char *s, exflib_uQword u[], long pr64)
{
    long e10;
    exflib_Qword sign, e2, shift, m_0;
    char deci[65536];
    exflib_uQword m[EXFLIB_CONST_MAX_PR], e_frac[EXFLIB_CONST_MAX_PR];
    long i;

    exflt_parse(s, deci, &sign, &e10);

    /* skip leading zero */
    for(i = 0; *(deci + i) == '0'; i++)
	e10--;

    /* all digit are '0' */
    if(*(deci + i) == '\0'){
	while(pr64 >= 0)
	    u[pr64--] = 0;
	return;
    }

    exflt_conv_radix(deci + i, m, pr64 + 2);

    exflib_convert_pow10(e10, &e2, e_frac, pr64 + 2);

    exflib_array_mul(m, m, e_frac, pr64 + 2);

    if(!m[0]){

	/* zero */
	u[0] = sign;

	for(i = 1; i <= pr64; i++)
	    u[i] = 0;

	return;
    }
    
    m_0 = (m[0] >> 1);

    for(shift = 0; m_0; shift++, m_0 >>= 1)
	continue;

    e2 += shift;

    if(e2 > 0x4000000000000000LL){
	/* over flow */

	u[0] = sign | (~EXFLIB_SIGN);
	for(i = 1; i <= pr64; i++)
	    u[i] = 0xffffffffffffffffULL;

    } else if(e2 < -EXFLIB_BIAS){

	/* under flow */

	u[0] = sign;
	for(i = 1; i <= pr64; i++)
	    u[i] = 0;

    } else {

	if(shift > 0) {
	    for(i = 1; i <= pr64; i++)
		u[i] = (m[i-1] << (64 - shift)) | (m[i] >> shift);
	} else {
	    for(i = 1; i <= pr64; i++)
		u[i] = m[i];
	}

	/* rounding */
	if(m[pr64 + 1] & EXFLIB_SIGN){

	    u[0] = 0;
	    for(i = pr64; u[i] == 0xffffffffffffffffULL; i--)
		u[i] = 0;

	    if(i == 0)
		e2++;
	    else
		u[i]++;

	}
	
	u[0] = sign | (e2 + EXFLIB_BIAS);
    }
    
    return;
}

/* End of atoef.c */
