/* -*- C -*- */
/* Time-stamp: "2006-02-20 22:36:20 fujiwara" */

/**********************************************************************/
/*  Copyright (C) 2000, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: double.c,v 1.1.1.1 2002/04/07 07:18:09 fujiwara Exp $";

#include "exfloat.h"
#include <stdio.h>	/* fprintf */

/**********************************************************************/
/* convert a exfloat to a double                                      */
/**********************************************************************/

double exflib_float_eftod(exflib_cuQword u[])
{
#if 0
    double d;
    exflib_uQword *pd = (exflib_uQword *)&d;
#else
    exflib_uQword pd;
    double *d;
#endif

    /* decompose exfloat format */
    exflib_Qword s = u[0] & EXFLIB_MASK_SGN;
    exflib_Qword e = u[0] & ~EXFLIB_MASK_SGN;
    exflib_uQword m = u[1];
    e -= EXFLIB_BIAS;

    /*                          +--------------------+
     *                          | double <-- exfloat |
     *                          +--------------------+
     *
     *                              Inf  <-- 2^(1023) * 1.fffffffffffff800
     *
     * 	maximum normalized number
     *      2^(1023) * 1.fffffffffffff   <-- 2^(1023) * 1.fffffffffffff7ff
     *                                         
     *	minimum normalized number
     *	    2^(-1022) * 1.0000000000000  <-- 2^(-1023) * 1.fffffffffffff000
     *
     *	maximum denormalized number
     *      2^(-1022) * 0.fffffffffffff
     *	  = 2^(-1023) * 1.ffffffffffffe  <-- 2^(-1023) * 1.ffffffffffffefff 
     *
     *	minimum denormalized number
     *      2^(-1022) * 0.0000000000001
     *	  = 2^(-1074) * 1.0              <-- 2^(-1075) * 1.8000000000000000
     *
     *                             zero  <-- 2^(-1075) * 1.7fffffffffffffff
     */

    /* infinity */
    if(e > 1023 || (e == 1023 && m >= 0xfffffffffffff800ULL)){
#if 0
	*pd = s | 0x7ff0000000000000ULL;
	return d;
#else
	pd = s | 0x7ff0000000000000ULL;
	d = (void *)&pd;
	return *d;
#endif	
    }

    /* normalized number */
    if(e > -1023 || (e == -1023 && m >= 0xfffffffffffff000ULL)){

	/* round off */
	if(m & 0x800){
	    m = (m + 0x1000) >> 12;
	    if(!m)
		e++;
	} else {
	    m >>= 12;
	}

#if 0
	*pd = s | (e + 0x3ff) << 52 | m;
	return d;
#else
	pd = s | (e + 0x3ff) << 52 | m;
	d = (void *)&pd;
	return *d;
#endif	
    }

    /* signed zero */
    if(e < -1075 || (e == -1075 && m <= 0x7fffffffffffffffULL)){
#if 0
	*pd = s;
	return d;
#else
	pd = s;
	d = (void *)&pd;
	return *d;
#endif	
    }

    /* denormalized number */
    m = (m >> 12) | 0x0010000000000000ULL;
    e += 0x3ff;

    while(e++)
	m = (m + 1) >> 1;

#if 0
    *pd = s | m;
    return d;
#else
    pd = s | m;
    d = (void *)&pd;
    return *d;
#endif
}

/**********************************************************************/
/* convert a double to a exfloat                                      */
/**********************************************************************/

void exflib_float_dtoef(const double d, exflib_uQword u[], long pr64)
{
    exflib_uQword s, e, m;
#if 0
    exflib_uQword *pd = (exflib_uQword *)&d;
#else
    exflib_uQword *pd = (void *)&d;
#endif    
    
    /* decompose IEEE754 double format */
    s = *pd & EXFLIB_MASK_SGN;
    e = (*pd & ~EXFLIB_MASK_SGN) >> 52;
    m = *pd << 12;

    /*
     *  IEEE754 double format  (0x7ff = 2047, 0x3ff = 1023)
     *
     *  (1) normalized number
     *      0 < e < 0x7ff --> (-1)^s * 2^(e - 0x3ff) * 1.m
     *
     *	(2) denormalized number
     *      e = 0, m != 0 --> (-1)^s * 2^(-0x3fe) * 0.m
     *
     *  (3) signed zero
     *      e = 0, m = 0 --> (-1)^s * 0.0
     *
     *  (4) infinity
     *      e = 0x7ff, m = 0 --> (-1)^s * Inf
     *
     *  (5) NaN
     *      s = u, e = 0x7ff, m != 0 --> NaN
     */
    
    /* normalized number */
    if(0 < e && e < 0x7ff){
	u[0] = s | (e - 0x3ff + EXFLIB_BIAS);
	u[1] = m;

	while(pr64 >= 2)
	    u[pr64--] = 0;

	return;
    }

    /* zero */
    if(e == 0 && m == 0){
	u[0] = s;

	while(pr64 >= 1)
	    u[pr64--] = 0;

	return;
    }

    /* Infinity */
    if(e == 0x7ff && m == 0){
	u[0] = s & ~EXFLIB_MASK_SGN;

	while(pr64 >= 1)
	    u[pr64--] = 0xffffffffffffffffULL;

	return;
    }

    /* denormalized number */
    if(e == 0 && m != 0){
	u[0] = s | (EXFLIB_BIAS - 0x3fe);

	while(!(m & EXFLIB_MASK_SGN)){
	    m <<= 1;
	    u[0]--;
	}

	u[0]--;
	u[1] = m << 1;

	while(pr64 >= 2)
	    u[pr64--] = 0;

	return;
    }
	
    /* NaN */
    fprintf(stderr, "Warning: Can't convert NaN to a exfloat number.\n");

    return;
}

#if 0
void exflib_mul_double(exflib_uQword u[], const double d, exflib_uQword prod[], long pr64)
{
    exflib_uQword *pd = (exflib_uQword *)&d;
    long i;
    exflib_Qword u_0;

    /* decompose IEEE754 double format */
    exflib_Qword s = *pd & EXFLIB_MASK_SGN;
    exflib_Qword e = (*pd & ~EXFLIB_MASK_SGN) >> 52;
    exflib_Qword m = *pd & 0x000fffffffffffffULL;

    /* normalized number */
    if(0 < e && e < 0x7ff){

	m |= 0x0010000000000000ULL;

	u_0 = u[0] & ~EXFLIB_MASK_SGN;
	prod[0] = EXFLIB_BIAS;
	for(i = 1; i <= pr64; i++)
	    prod[i] = u[i];

	exflib_i32mul(prod, m, prod, pr64);		/* Not overflow */

	e = (e - 0x3ff - 52) + (prod[0] - EXFLIB_BIAS) + u_0;
	s ^= u_0 & EXFLIB_MASK_SGN;

	if(e > 0){
	    prod[0] = s | e;
	    return;
	}

	/* under flow */
	if(e + 0x3ff + 52 >= 0){
	    prod[0] = s;

	    while(pr64 >= 1)
		prod[pr64--] = 0;

	    return;
	}

	/* over flow */
	prod[0] = s | ~EXFLIB_MASK_SGN;

	while(pr64 >= 1)
	    prod[pr64--] = 0xffffffffffffffffULL;

	return;
    }

    /* signed zero */
    if(e == 0 && m == 0){
	prod[0] = s ^ (u[0] & EXFLIB_MASK_SGN);

	while(pr64 >= 1)
	    prod[pr64--] = 0;

	return;
    }

    /* infinity */
    if(e == 0x7ff && m != 0){
	prod[0] = (s ^ (u[0] & EXFLIB_MASK_SGN))| ~EXFLIB_MASK_SGN;

	while(pr64 >= 1)
	    prod[pr64--] = 0xffffffffffffffffULL;

	return;
    }

    /* denormalized number */
    if(e == 0 && m != 0){

	u_0 = u[0] & ~EXFLIB_MASK_SGN;
	prod[0] = EXFLIB_BIAS;
	for(i = 1; i <= pr64; i++)
	    prod[i] = u[i];

	exflib_i32mul(prod, m, prod, pr64);		/* not overflow */

	s ^= u_0 & EXFLIB_MASK_SGN;
	e = (-0x3fe - 52) + (prod[0] - EXFLIB_BIAS) + u_0;

	/* underflow */
	if(e < 0){

	    prod[0] = s;

	    while(pr64 >= 1)
		prod[pr64--] = 0;

	    return;
	}

	prod[0] = s | m;

	return;
    }

    /* NaN */
    fprintf(stderr, "Warning: Can't multiply NaN to a exfloat number.\n");

    return;
}
#endif

/* End of double.c */
