/* -*- C -*- */
/* Time-stamp: "2006-02-27 22:37:59 fujiwara" */

/**********************************************************************/
/*  Copyright (C) 1999, Fujiwara Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: expression.c,v 1.1.1.1 2002/04/07 07:18:09 fujiwara Exp $";

#include <stdio.h>
#include <stdlib.h>
#include "exfloat.h"

/* prototype declaration */
void exflib_float_atoef(const char *s, exflib_uQword u[], long pr64);
void exflib_float_htoef(const char *s, exflib_uQword u[], long pr64);
void exflib_float_ctoef(const char *s, exflib_uQword u[], long pr64);

static char expression(exflib_uQword *u);
static char term(exflib_uQword *u);
static char unary(exflib_uQword *u);
static char token(exflib_uQword *u);

/**********************************************************************/
/* global variable                                                    */
/**********************************************************************/

static const char *ptr;
static long stpr64;

/**********************************************************************/
/* expression parser                                                  */
/**********************************************************************/

static char expression(exflib_uQword *u)
{
    exflib_uQword v[1024];
    char last_token;

    last_token = term(u);

    while(last_token){

	switch(last_token){

	  case '+':
	    last_token = term(v);
	    exflib_float_add(u, u, v, stpr64);
	    break;

	  case '-':
	    last_token = term(v);
	    exflib_float_sub(u, u, v, stpr64);
	    break;

	  default:
	    return last_token;
	}
    }
    return last_token;
}

/**********************************************************************/
/* term parser                                                        */
/**********************************************************************/

static char term(exflib_uQword *u)
{
    exflib_uQword v[1024];
    char last_token;

    last_token = unary(u);

    while(last_token){

	switch(last_token){

	  case '*':
	    last_token = unary(v);
	    exflib_float_mul(u, u, v, stpr64);
	    break;

	  case '/':
	    last_token = unary(v);
	    exflib_float_div(u, u, v, stpr64);
	    break;

	  default:
	    return last_token;
	}
    }
    return last_token;
}

/**********************************************************************/
/* unary parser                                                       */
/**********************************************************************/

static char unary(exflib_uQword *u)
{
    exflib_uQword v[1024];
    char last_token;

    last_token = token(u);

    switch(last_token){

      case 'v':
	last_token = token(v);
	break;

      case '+':
	last_token = unary(u);
	break;
 
      case '-':
	last_token = unary(u);
	u[0] = u[0] ^ EXFLIB_SIGN;
	break;

      case '(':
	if((last_token = expression(u)) != ')'){
	    fprintf(stderr, "Too many ('s.\n");
	    exit(1);
	}
	last_token = token(v);
	break;
    }

    return last_token;
}

/**********************************************************************/
/* tokenizer                                                          */
/**********************************************************************/

static char token(exflib_uQword *u)
{
    char last_token;
    char buf[1024];
    int i;

    if(!(*ptr)){

	last_token = (char)NULL;

    } else if(*ptr == '+' || *ptr == '-' || *ptr == '*' || *ptr == '/'
	      || *ptr == '(' || *ptr == ')') {

	last_token = *ptr++;

    } else {

	/* value */

	switch(*ptr){

	  case '#':

	    ptr++;

	    /* constant identifier */
	    
	    i = 0;
	    while(('A' <= *ptr && *ptr <= 'Z') || (*ptr == '_') ||
		  ('a' <= *ptr && *ptr <= 'z') || ('0' <= *ptr && *ptr <= '9'))
		buf[i++] = *ptr++;
	    buf[i] = '\0';

	    exflib_float_ctoef(buf, u, stpr64);
	    
	    break;

	  case '@':

	    ptr++;
	    
	    /* memory dump */
	    
	    i = 0;
	    while(('0' <= *ptr && *ptr <= '9')
		  || ('a' <= *ptr && *ptr <= 'f')
		  || ('A' <= *ptr && *ptr <= 'F')
		  || (*ptr == ':'))
		buf[i++] = *ptr++;
	    buf[i] = '\0';
	    
	    exflib_float_htoef(buf, u, stpr64);

	    break;

	  default:

	    /* decimal format */
	    i = 0;
	    while(('0' <= *ptr && *ptr <= '9') || (*ptr == '.'))
		buf[i++] = *ptr++;

	    if(*ptr == 'E' || *ptr == 'e'){

		buf[i++] = *ptr++;

		if(*ptr == '+' || *ptr == '-')
		    buf[i++] = *ptr++;

		while('0' <= *ptr && *ptr <= '9')
		    buf[i++] = *ptr++;
	    }

	    buf[i] = '\0';
	    
	    exflib_float_atoef(buf, u, stpr64);
	    
	    break;
	}

	last_token = 'v';
    }

    return last_token;
}

/**********************************************************************/
/* assign exfloat by string                                           */
/* convert a expression to a exfloat                                  */
/**********************************************************************/

void exflib_float_etoef(const char *s, exflib_uQword u[], long pr64)
{
    ptr = s;
    stpr64 = pr64;

    expression(u);

    return;
}

/* End of expression.c */
