/* -*- C -*- */
/* Time-stamp: "2006-01-16 10:01:44 fujiwara" */

/**********************************************************************/
/*  Copyright (C) 1999-2005, FUJIWARA Hiroshi                         */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: format.c,v 1.1.1.1 2002/04/07 07:18:09 fujiwara Exp $";

#include "exfloat.h"

/*****************************************/
/* size of u      is pr64 + 1,           */
/* size of target is pr64 + 2.           */
/*****************************************/	

void exflib_convert_pow(exflib_cuQword u[], exflib_Qword *d_int, exflib_uQword target[], long pr64)
{
    long top, size;
    exflib_uQword d_frac[EXFLIB_CONST_MAX_PR], y[EXFLIB_CONST_MAX_PR], term[EXFLIB_CONST_MAX_PR];

    exflib_uQword log_10_2[] = {
#include "log_10_2.fixed"
    };
    exflib_uQword ln10[] = {
#include "ln10.fixed"
    };

    exflib_Qword ui;
    long i;

    size = pr64 + 2;

    /***************************************************/
    /* convert 2^e                                     */
    /***************************************************/
    
    if((ui = u[0] & ~EXFLIB_MASK_SGN) >= EXFLIB_BIAS){

	exflib_array_imul(d_frac, log_10_2, ui - EXFLIB_BIAS, size);
	*d_int = d_frac[0];

    } else {

	exflib_array_imul(d_frac, log_10_2, EXFLIB_BIAS - ui, size);
	*d_int = -d_frac[0] - 1;

	/* bit reverse & add 1 ulp */
	for(i = 1; i < size; i++)
	    d_frac[i] = ~d_frac[i];
	for(i = size - 1; d_frac[i] == 0xffffffffffffffffLL; i--)
	    d_frac[i] = 0;
	d_frac[i]++;
    }
    d_frac[0] = 0;

    /***************************************************/
    /* calculate 10^(d_frac)                           */
    /***************************************************/

    exflib_array_mul(y, ln10, d_frac, size);

    for(i = 0; i < size; i++)
	target[i] = term[i] = y[i];
    target[0]++;

    for(top = 0, i = 2; top < size; i++){		/* Taylor expansion */
	exflib_array_mul(term, term, y, size);
	if((top > 0) && (term[top - 1] != 0))
	    top--;
	exflib_array_idiv(term, term, i, size, &top);
	exflib_array_add(target, target, term, size);
    }

    /***************************************************/
    /* 1.F * d^(dfrac)                                 */
    /***************************************************/

    y[0] = 1;
    for(i = 1; i <= pr64; i++)
	y[i] = u[i];
    y[i] = 0;
    exflib_array_mul(target, target, y, size);

    /***************************************************/
    /* normalize                                       */
    /***************************************************/

    if(target[0] >= 10){
	top = 0;
	exflib_array_idiv(target, target, 10ULL, size, &top);
	*d_int += 1;
    }

    return;
}

/***********************************************************************/
/* convert radix 2^64 to 10                                            */
/*                                                                     */
/* return value is pointer to the next char                            */
/***********************************************************************/

static char *exflt_hex2dec(exflib_uQword u[], long usize, char d[], exflib_Qword dsize)
{
    exflib_uQword u_0;
    long pr_cntr, cntr, i;

    for(pr_cntr = 0; pr_cntr + 18 < dsize; pr_cntr += 19){

	u[0] = 0;
	exflib_array_imul(u, u, EXFLIB_TEN_19, usize);

	u_0 = u[0];

	for(i = 18; i >= 0; i--){
	    *(d + i) = u_0%10 + '0';
	    u_0 /= 10;
	}
	d += 19;
    }

    for(u_0 = 1, cntr = 0; pr_cntr < dsize; pr_cntr++, cntr++)
	u_0 *= 10;

    if(cntr > 0){

	u[0] = 0;
	exflib_array_imul(u, u, u_0, usize);
	u_0 = u[0];

	for(i = cntr - 1; i >= 0; i--){
	    *(d + i) = u_0%10 + '0';
	    u_0 /= 10;
	}
	d += cntr;
    }

    return d;
}

/***********************************************************************/
/* scientific format                                                   */
/***********************************************************************/

void exflib_scientific_format(exflib_uQword flag, exflib_uQword u[],
			     exflib_Qword u_pow, char output_string[])
{
    long cntr;
    char pow_deci[32];

    /* set local variables */
    long hexpr = flag & 0xffff;
    long decpr = (flag >> 16) & 0xffff;

    /***************************************************/
    /* make output string                              */
    /***************************************************/

    if(flag & EXFLIB_SIGN)	*output_string++ = '-';

    *output_string++ = u[0] + '0';	*output_string++ = '.';

    /***************************************************/
    /* fraction part                                   */
    /***************************************************/

    output_string = exflt_hex2dec(u, hexpr, output_string, decpr + 1);

    /***************************************************/
    /* rounding                                        */
    /***************************************************/
    if(*--output_string >= '5'){

	cntr = 0;
	while(*(output_string - 1) == '9'){
	    *--output_string = '0';
	    cntr++;
	}

	if(*(output_string - 1) == '.'){
	    if(*(output_string - 2) == '9'){
		*(output_string - 2) = '1';
		u_pow += 1;
	    } else {
		*(output_string - 2) += 1;
	    }

	} else {
	    *(output_string - 1) += 1;
	}
	output_string += cntr;
    }

    /***************************************************/
    /* erase a decimal point & following '0'           */
    /***************************************************/
    if(decpr == 0){
	output_string--;
    } else if(flag & EXFLIB_G_FORMAT){
	while(*(output_string - 1) == '0')
	    output_string--;
	if(*(output_string - 1) == '.')
	    output_string--;
    }

    /***************************************************/
    /* exponent part                                   */
    /***************************************************/

    if(flag & EXFLIB_CAPITAL)
	*output_string++ = 'E';
    else
	*output_string++ = 'e';
    
    *output_string++ = '+';

    if(u_pow < 0){
	*(output_string - 1) = '-';
	u_pow = -u_pow;
    }

    cntr = 0;

    do {
	pow_deci[cntr++] = u_pow%10 + '0';
	u_pow /= 10;
    } while(u_pow > 0);

    for(cntr--; cntr >= 0; cntr--)
	*output_string++ = pow_deci[cntr];

    *output_string = '\0';

    return;
}

/***********************************************************************/
/* fixed format                                                        */
/***********************************************************************/

void exflib_fixed_format(exflib_uQword flag, exflib_uQword u[],
			exflib_Qword u_pow, char output_string[])
{
    char unformatted[65536], *d;
    long cntr;
    
    /* set local variables */
    long hexpr = flag & 0xffff;
    long decpr = (flag >> 16) & 0xffff;

    /***************************************************/
    /* small for output                                */
    /***************************************************/

    if(u_pow <= - decpr - 1){

	if(flag & EXFLIB_SIGN)	*output_string++ = '-';

	*output_string++ =  '0';	*output_string++ = '.';

	for(cntr = 0; cntr < decpr - 1; cntr++)
	    *output_string++ = '0';

	if(cntr < decpr){

	    if(u_pow == -decpr - 1)
		*output_string++ = (u[0] >= 5) + '0';
	    else
		*output_string++ = '0';
	}

    } else {

	/***************************************************/
	/* large enough for output                         */
	/***************************************************/

	d = unformatted;
	*d++ = u[0] + '0';
	d = exflt_hex2dec(u, hexpr, d, decpr + u_pow + 1);

	/***************************************************/
	/* rounding                                        */
	/***************************************************/

	if(*--d >= '5'){

	    /* set last digit in case carry-up reaches the top digit */
	    *d-- = '0';

	    while((d > unformatted) && (*d == '9'))
		*d-- = '0';

	    if((d == unformatted) && (*d == '9')){

		/* carry-up reaches the top digit */
		*d = '1';
		*(d + 1) = '0';

		u_pow += 1;

	    } else {
		*d += 1;
	    }
	}

	/***************************************************/
	/* format                                          */
	/***************************************************/

	d = unformatted;
    
	if(flag & EXFLIB_SIGN)    *output_string++ = '-';

	if(u_pow >= 0){

	    for(cntr = 0; cntr <= u_pow; cntr++)
		*output_string++ = *d++;
	    
	    *output_string++ = '.';
	    
	    for(cntr = 0; cntr < decpr; cntr++)
		*output_string++ = *d++;

	} else {

	    *output_string++ = '0';	*output_string++ = '.';

	    for(cntr = 1; cntr < -u_pow; cntr++)
		*output_string++ = '0';

	    while(cntr++ <= decpr)
		*output_string++ = *d++;
	}
    }
    /* end of large number format */

    /***************************************************/
    /* erase a decimal point and zeros.                */
    /***************************************************/

    if(flag & EXFLIB_G_FORMAT){

	while(*--output_string == '0')
	    continue;

	output_string++;
    }

    if(*(output_string - 1) == '.')
	output_string--;

    *output_string = '\0';

    return;
}

/* End of format.c */
