/* -*- C -*- */
/* Time-stamp: "2006-03-17 18:58:49 fujiwara" */

/**********************************************************************/
/*  Copyright (C) 2006, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: fortran_format.c,v 1.1.1.1 2002/04/07 07:18:09 fujiwara Exp $";

#ifdef __BORLANDC__
typedef const            __int64 exflib_cQword;
typedef const unsigned   __int64 exflib_cuQword;
typedef       unsigned   __int64 exflib_uQword;
typedef                  __int64 exflib_Qword;
typedef       unsigned      long exflib_ulong;
#else
typedef const          long long exflib_cQword;
typedef const unsigned long long exflib_cuQword;
typedef       unsigned long long exflib_uQword;
typedef                long long exflib_Qword;
typedef       unsigned      long exflib_ulong;
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/*----------------------------------------------------------------------*/
/* format exfloat number in FORTRAN manner                              */
/*----------------------------------------------------------------------*/

static exflib_cuQword EXFLIB_F_FORMAT = 0x0400000000000000ULL;
static exflib_cuQword EXFLIB_E_FORMAT = 0x0200000000000000ULL;
static exflib_cuQword EXFLIB_SIGN     = 0x8000000000000000ULL;

static const long exflib_fortran_output_default_width = 6;
static const long exflib_fortran_output_default_precision = 6;

#define FORTRAN_PADDING_CHAR ' '

void exflib_trim_c_null(char *str);

void exflib_convert_pow(exflib_cuQword u[], exflib_Qword *d_int,
			exflib_uQword target[], long pr64);
void exflib_scientific_format(exflib_uQword flag, exflib_uQword u[],
			      exflib_Qword u_pow, char output_string[]);
void exflib_fixed_format(exflib_uQword flag, exflib_uQword u[],
			 exflib_Qword u_pow, char output_string[]);

void exflib_float_fortran_format(char *const dst,
				 const char *const form,
				 exflib_cuQword src[],
				 const size_t pr64,
				 const size_t dst_len)
{
    long i, ii;
    long width, precision;
    char buf[1024];

    exflib_uQword b64[pr64 + 2];	/* malloc ˾ޤ */
    exflib_Qword deci_pow;

    /* ν */
    memset(dst, FORTRAN_PADDING_CHAR, dst_len);
    dst[0] = '\0';
    buf[0] = '\0';

    /**************************************************/
    /* format check                                   */
    /* Hexadecimal output, or unkown specifier        */
    /**************************************************/
    if((form[0] != 'E') && (form[0] != 'e')
       && (form[0] != 'F') && (form[0] != 'f')){

	for(i = 0; i <= pr64; i++){
	    sprintf(buf, "%016llx", src[i]);
	    strcat(dst, buf);
	    if(i < pr64)
		strcat(dst, ":");
	}
	exflib_trim_c_null(dst);
	return;
    }

    /*****************************************************/
    /* Format is (Ew.p) or (ew.p) or (Fw.p) or (fw.p)    */
    /*****************************************************/
    /* counting width */
    width = 0;
    for(i = 1; form[i] && ('0' <= form[i]) && (form[i] <= '9') ; i++){
	width *= 10;
	width += form[i] - '0';
    }
    if(i == 1)
	width = exflib_fortran_output_default_width;

    if((form[i] != '.') && (form[i] != '\0')){
	sprintf(dst, "Unkown Format Instruction : %s", form);
	exflib_trim_c_null(dst);
	return;
    }
    if(form[i])
        i++;	/* skip the period */
	
    /* counting precision */
    precision = 0;
    ii = i;
    for(; form[i] && ('0' <= form[i]) && (form[i] <= '9') ; i++){
	precision *= 10;
	precision += form[i] - '0';
    }
    if(ii == i)
	precision = exflib_fortran_output_default_precision;

    if(width < precision)
	width = precision;

    /**************************************************/
    /* special numbers                                */
    /**************************************************/
    if(src[0] == 0){

	long printed = 1;
	strcat(buf, "0");
	if(precision > 0){
	    strcat(buf, "."); printed++;
	    for(i = 0; i < precision; i++, printed++)
		strcat(buf, "0");
	}

	i = width - printed;
	while(i-- > 0)
	    strcat(dst, " ");

	strcat(dst, buf);
	exflib_trim_c_null(dst);

	return;
	
    } else if(src[0] == 0x8000000000000000ULL){
	
	long printed = 2;
	strcat(buf, "-0");
	if(precision > 0){
	    strcat(buf, "."); printed++;
	    for(i = 0; i < precision; i++, printed++)
		strcat(buf, "0");
	}

	i = width - printed;
	while(i-- > 0)
	    strcat(dst, " ");

	strcat(dst, buf);
	exflib_trim_c_null(dst);

	return;

    } else if(src[0] == 0x7fffffffffffffffULL){

	long printed = strlen("Infty");
	i = width - printed;
	while(i-- > 0)
	    strcat(dst, " ");
	strcat(dst, "Infty");
	exflib_trim_c_null(dst);

	return;

    } else if(src[0] == 0xffffffffffffffffULL){

	long printed = strlen("-Infty");
	i = width - printed;
	while(i-- > 0)
	    strcat(dst, " ");
	strcat(dst, "-Infty");
	exflib_trim_c_null(dst);

	return;
    }

    /*---------------------------------------------------------------*/
    /* 1.(num) * 2^e --> i.(b64) * 10^d ('i' is 1 digit in radix 10) */
    /*---------------------------------------------------------------*/
    exflib_convert_pow(src, &deci_pow, b64, pr64);

    /**************************************************/
    /* scientific format                              */
    /**************************************************/
    if((form[0] == 'E') || (form[0] == 'e')){

	exflib_uQword flag;
	
        flag = (src[0] & EXFLIB_SIGN) | EXFLIB_E_FORMAT
            | (precision << 16) | (pr64 + 2);
        
        exflib_scientific_format(flag, b64, deci_pow, buf);
    }
    
    /**************************************************/
    /* fixed point format                             */
    /**************************************************/
    if((form[0] == 'F') || (form[0] == 'f')){

	exflib_uQword flag;

        flag = (src[0] & EXFLIB_SIGN) | EXFLIB_F_FORMAT
            | (precision << 16) | (pr64 + 2);

        exflib_fixed_format(flag, b64, deci_pow, buf);
    }

    i = width - strlen(buf);
    while(i-- > 0)
	strcat(dst, " ");
    strcat(dst, buf);
    exflib_trim_c_null(dst);
    return;
}
    
/*************************************************************/
/* interface (arguments conversion)                          */
/*************************************************************/

void exflib_float_fortran_format_f77_(char *const dst,
				      const char *const form,
				      exflib_cuQword src[],
				      const size_t *pr64,
				      const size_t *dst_len)
{
    exflib_float_fortran_format(dst, form, src, *pr64, *dst_len);
    return;
}

void exflib_float_fortran_format_f77__(char *const dst,
				       const char *const form,
				       exflib_cuQword src[],
				       const size_t *pr64,
				       const size_t *dst_len)
{
    exflib_float_fortran_format(dst, form, src, *pr64, *dst_len);
    return;
}

void exflib_float_fortran_format_f90_(char *dst,
				      const char *const form,
				      exflib_cuQword **src,
				      const size_t *pr64,
				      const size_t *dst_len)
{
    exflib_float_fortran_format(dst, form, *src, *pr64, *dst_len);
    return;
}

void exflib_float_fortran_format_f90__(char *dst,
				       const char *const form,
				       exflib_cuQword **src,
				       const size_t *pr64,
				       const size_t *dst_len)
{
    exflib_float_fortran_format(dst, form, *src, *pr64, *dst_len);
    return;
}

/* End of file */
