/* -*- C -*- */
/* Time-stamp: "2006-03-17 16:48:52 fujiwara" */

/**********************************************************************/
/*  Mathematical functions interfaces for FORTRAN                     */
/*  Copyright (C) 2006, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: c2f.c,v 1.1.1.1 2002/04/07 07:18:09 fujiwara Exp $";

#include "exfloat.h"

/* fabs ------------------------------------------------------------- */

void exflib_float_fabs_f77_(exflib_uQword dst[], exflib_cuQword u[],
			    const long *pr64){
  int i;
  dst[0] = u[0] & 0x7fffffffffffffffULL;
  for(i = 1; i < *pr64; i++)
    dst[i] = u[i];
  return;
}

void exflib_float_fabs_f77__(exflib_uQword dst[], exflib_cuQword u[],
			     const long *pr64){
  int i;
  dst[0] = u[0] & 0x7fffffffffffffffULL;
  for(i = 1; i < *pr64; i++)
    dst[i] = u[i];
  return;
}

void exflib_float_fabs_f90_(exflib_uQword **p_dst, exflib_cuQword **p_u,
			    const long *pr64){
  int i;
  exflib_uQword *dst = p_dst[0];
  exflib_cuQword *u = p_u[0];

  dst[0] = u[0] & 0x7fffffffffffffffULL;
  for(i = 1; i < *pr64; i++)
    dst[i] = u[i];
  return;
}

void exflib_float_fabs_f90__(exflib_uQword **p_dst, exflib_cuQword **p_u,
			     const long *pr64){
  int i;
  exflib_uQword *dst = p_dst[0];
  exflib_cuQword *u = p_u[0];

  dst[0] = u[0] & 0x7fffffffffffffffULL;
  for(i = 1; i < *pr64; i++)
    dst[i] = u[i];
  return;
}

/* sqrt ------------------------------------------------------------- */
/* exflib_ulong exflib_float_sqrt(exflib_uQword y[], exflib_cuQword x[], long pr64, exflib_uQword w[]); */

exflib_ulong exflib_float_sqrt_f77_(exflib_uQword *dst, exflib_cuQword *src,
				    long *pr64, exflib_uQword *mem)
{
    return exflib_float_sqrt(dst, src, *pr64, mem);
}

exflib_ulong exflib_float_sqrt_f77__(exflib_uQword *dst, exflib_cuQword *src,
				     long *pr64, exflib_uQword *mem)
{
    return exflib_float_sqrt(dst, src, *pr64, mem);
}

exflib_ulong exflib_float_sqrt_f90_(exflib_uQword **dst, exflib_cuQword **src,
				    long *pr64, exflib_uQword **mem)
{
    return exflib_float_sqrt(*dst, *src, *pr64, *mem);
}

exflib_ulong exflib_float_sqrt_f90__(exflib_uQword **dst, exflib_cuQword **src,
				     long *pr64, exflib_uQword **mem)
{
    return exflib_float_sqrt(*dst, *src, *pr64, *mem);
}

/* sin ------------------------------------------------------------- */
exflib_ulong exflib_float_sin_f77_(exflib_uQword *dst, exflib_cuQword *src,
				   long *pr64, exflib_uQword *mem)
{
    return exflib_float_sin(dst, src, *pr64, mem);
}

exflib_ulong exflib_float_sin_f77__(exflib_uQword *dst, exflib_cuQword *src,
				    long *pr64, exflib_uQword *mem)
{
    return exflib_float_sin(dst, src, *pr64, mem);
}

exflib_ulong exflib_float_sin_f90_(exflib_uQword **dst, exflib_cuQword **src,
				   long *pr64, exflib_uQword **mem)
{
    return exflib_float_sin(*dst, *src, *pr64, *mem);
}

exflib_ulong exflib_float_sin_f90__(exflib_uQword **dst, exflib_cuQword **src,
				    long *pr64, exflib_uQword **mem)
{
    return exflib_float_sin(*dst, *src, *pr64, *mem);
}

/* cos ------------------------------------------------------------- */
exflib_ulong exflib_float_cos_f77_(exflib_uQword *dst, exflib_cuQword *src,
                                   long *pr64, exflib_uQword *mem)
{
    return exflib_float_cos(dst, src, *pr64, mem);
}

exflib_ulong exflib_float_cos_f77__(exflib_uQword *dst, exflib_cuQword *src,
				    long *pr64, exflib_uQword *mem)
{
    return exflib_float_cos(dst, src, *pr64, mem);
}

exflib_ulong exflib_float_cos_f90_(exflib_uQword **dst, exflib_cuQword **src,
				   long *pr64, exflib_uQword **mem)
{
    return exflib_float_cos(*dst, *src, *pr64, *mem);
}

exflib_ulong exflib_float_cos_f90__(exflib_uQword **dst, exflib_cuQword **src,
				    long *pr64, exflib_uQword **mem)
{
    return exflib_float_cos(*dst, *src, *pr64, *mem);
}

/* exp ------------------------------------------------------------ */
exflib_ulong exflib_float_exp_f77_(exflib_uQword *dst, exflib_cuQword *src,
                                   long *pr64, exflib_uQword *mem)
{
    return exflib_float_exp(dst, src, *pr64, mem);
}

exflib_ulong exflib_float_exp_f77__(exflib_uQword *dst, exflib_cuQword *src,
				    long *pr64, exflib_uQword *mem)
{
    return exflib_float_exp(dst, src, *pr64, mem);
}

exflib_ulong exflib_float_exp_f90_(exflib_uQword **dst, exflib_cuQword **src,
				   long *pr64, exflib_uQword **mem)
{
    return exflib_float_exp(*dst, *src, *pr64, *mem);
}

exflib_ulong exflib_float_exp_f90__(exflib_uQword **dst, exflib_cuQword **src,
				    long *pr64, exflib_uQword **mem)
{
    return exflib_float_exp(*dst, *src, *pr64, *mem);
}

/* log ------------------------------------------------------------ */
exflib_ulong exflib_float_log_f77_(exflib_uQword *dst, exflib_cuQword *src,
                                   long *pr64, exflib_uQword *mem, exflib_uQword *table)
{
    return exflib_float_log(dst, src, *pr64, mem, table);
}

exflib_ulong exflib_float_log_f77__(exflib_uQword *dst, exflib_cuQword *src,
				    long *pr64, exflib_uQword *mem, exflib_uQword *table)
{
    return exflib_float_log(dst, src, *pr64, mem, table);
}

exflib_ulong exflib_float_log_f90_(exflib_uQword **dst, exflib_cuQword **src,
				   long *pr64, exflib_uQword **mem, exflib_uQword **table)
{
    return exflib_float_log(*dst, *src, *pr64, *mem, *table);
}

exflib_ulong exflib_float_log_f90__(exflib_uQword **dst, exflib_cuQword **src,
				    long *pr64, exflib_uQword **mem, exflib_uQword **table)
{
    return exflib_float_log(*dst, *src, *pr64, *mem, *table);
}

/* pow ------------------------------------------------------------ */

exflib_ulong exflib_float_pow_f77_(exflib_uQword *dst,
				   exflib_cuQword *base, exflib_cuQword *p,
				   long *pr64, exflib_uQword *mem, exflib_uQword *table)
{
    return exflib_float_pow(dst, base, p, *pr64, mem, table);
}

exflib_ulong exflib_float_pow_f77__(exflib_uQword *dst,
				    exflib_cuQword *base, exflib_cuQword *p,
				    long *pr64, exflib_uQword *mem, exflib_uQword *table)
{
    return exflib_float_pow(dst, base, p, *pr64, mem, table);
}

exflib_ulong exflib_float_pow_f90_(exflib_uQword **dst,
				   exflib_cuQword **base, exflib_cuQword **p,
				   long *pr64, exflib_uQword **mem, exflib_uQword **table)
{
    return exflib_float_pow(*dst, *base, *p, *pr64, *mem, *table);
}

exflib_ulong exflib_float_pow_f90__(exflib_uQword **dst,
				    exflib_cuQword **base, exflib_cuQword **p,
				    long *pr64, exflib_uQword **mem, exflib_uQword **table)
{
    return exflib_float_pow(*dst, *base, *p, *pr64, *mem, *table);
}

/* pow (long) ------------------------------------------------------ */

exflib_ulong exflib_float_pow_long_f77_(exflib_uQword *dst,
					exflib_cuQword *base, exflib_Qword *p,
					long *pr64, exflib_uQword *mem)
{
    return exflib_float_pow_long(dst, base, *p, *pr64, mem);
}

exflib_ulong exflib_float_pow_long_f77__(exflib_uQword *dst,
					 exflib_cuQword *base, exflib_Qword *p,
					 long *pr64, exflib_uQword *mem)
{
    return exflib_float_pow_long(dst, base, *p, *pr64, mem);
}

exflib_ulong exflib_float_pow_long_f90_(exflib_uQword **dst,
					exflib_cuQword **base, exflib_Qword *p,
					long *pr64, exflib_uQword **mem)
{
    return exflib_float_pow_long(*dst, *base, *p, *pr64, *mem);
}

exflib_ulong exflib_float_pow_long_f90__(exflib_uQword **dst,
					 exflib_cuQword **base, exflib_Qword *p,
					 long *pr64, exflib_uQword **mem)
{
    return exflib_float_pow_long(*dst, *base, *p, *pr64, *mem);
}

/* asin ------------------------------------------------------------ */

exflib_ulong exflib_float_asin_f77_(exflib_uQword *dst, exflib_cuQword *src,
				    long *pr64, exflib_uQword *mem)
{
    return exflib_float_asin(dst, src, *pr64, mem);
}

exflib_ulong exflib_float_asin_f77__(exflib_uQword *dst, exflib_cuQword *src,
				     long *pr64, exflib_uQword *mem)
{
    return exflib_float_asin(dst, src, *pr64, mem);
}

exflib_ulong exflib_float_asin_f90_(exflib_uQword **dst, exflib_cuQword **src,
				    long *pr64, exflib_uQword **mem)
{
    return exflib_float_asin(*dst, *src, *pr64, *mem);
}

exflib_ulong exflib_float_asin_f90__(exflib_uQword **dst, exflib_cuQword **src,
				     long *pr64, exflib_uQword **mem)
{
    return exflib_float_asin(*dst, *src, *pr64, *mem);
}

/* atan ------------------------------------------------------------ */

exflib_ulong exflib_float_atan_f77_(exflib_uQword *dst, exflib_cuQword *src,
				    long *pr64, exflib_uQword *mem, exflib_uQword *table)
{
    return exflib_float_atan(dst, src, *pr64, mem, table);
}

exflib_ulong exflib_float_atan_f77__(exflib_uQword *dst, exflib_cuQword *src,
				     long *pr64, exflib_uQword *mem, exflib_uQword *table)
{
    return exflib_float_atan(dst, src, *pr64, mem, table);
}

exflib_ulong exflib_float_atan_f90_(exflib_uQword **dst, exflib_cuQword **src,
				    long *pr64, exflib_uQword **mem, exflib_uQword **table)
{
    return exflib_float_atan(*dst, *src, *pr64, *mem, *table);
}

exflib_ulong exflib_float_atan_f90__(exflib_uQword **dst, exflib_cuQword **src,
				     long *pr64, exflib_uQword **mem, exflib_uQword **table)
{
    return exflib_float_atan(*dst, *src, *pr64, *mem, *table);
}

/* modf ------------------------------------------------------------ */

void exflib_float_modf_f77_(exflib_cuQword *src,
			    exflib_uQword *ip, exflib_uQword *fp, long *pr64)
{
    exflib_float_modf(src, ip, fp, *pr64);
    return;
}

void exflib_float_modf_f77__(exflib_cuQword *src,
			    exflib_uQword *ip, exflib_uQword *fp, long *pr64)
{
    exflib_float_modf(src, ip, fp, *pr64);
    return;
}

void exflib_float_modf_f90_(exflib_cuQword **src,
			    exflib_uQword **ip, exflib_uQword **fp, long *pr64)
{
    exflib_float_modf(*src, *ip, *fp, *pr64);
    return;
}

void exflib_float_modf_f90__(exflib_cuQword **src,
			    exflib_uQword **ip, exflib_uQword **fp, long *pr64)
{
    exflib_float_modf(*src, *ip, *fp, *pr64);
    return;
}


/* fmod ------------------------------------------------------------ */

void exflib_float_fmod_f77_(exflib_uQword *r,
			    exflib_cuQword *d, exflib_cuQword *m, long *pr64,
			    exflib_uQword *mem)
{
    exflib_float_fmod(r, d, m, *pr64, mem);
    return;
}

void exflib_float_fmod_f77__(exflib_uQword *r,
			     exflib_cuQword *d, exflib_cuQword *m, long *pr64,
			     exflib_uQword *mem)

{
    exflib_float_fmod(r, d, m, *pr64, mem);
    return;
}

void exflib_float_fmod_f90_(exflib_uQword **r,
			    exflib_cuQword **d, exflib_cuQword **m, long *pr64,
			    exflib_uQword **mem)
{
    exflib_float_fmod(*r, *d, *m, *pr64, *mem);
    return;
}

void exflib_float_fmod_f90__(exflib_uQword **r,
			     exflib_cuQword **d, exflib_cuQword **m, long *pr64,
			     exflib_uQword **mem)
{
    exflib_float_fmod(*r, *d, *m, *pr64, *mem);
    return;
}

/* ceil ------------------------------------------------------------ */

void exflib_float_ceil_f77_(exflib_uQword *ip,
			    exflib_cuQword *x, long *pr64,
			    exflib_uQword *mem)
{
    exflib_float_ceil(ip, x, *pr64, mem);
    return;
}

void exflib_float_ceil_f77__(exflib_uQword *ip,
			     exflib_cuQword *x, long *pr64,
			    exflib_uQword *mem)
{
    exflib_float_ceil(ip, x, *pr64, mem);
    return;
}

void exflib_float_ceil_f90_(exflib_uQword **ip,
			    exflib_cuQword **x, long *pr64,
			    exflib_uQword **mem)
{
    exflib_float_ceil(*ip, *x, *pr64, *mem);
    return;
}

void exflib_float_ceil_f90__(exflib_uQword **ip,
			     exflib_cuQword **x, long *pr64,
			     exflib_uQword **mem)
{
    exflib_float_ceil(*ip, *x, *pr64, *mem);
    return;
}

/* floor ------------------------------------------------------------ */

void exflib_float_floor_f77_(exflib_uQword *ip,
			     exflib_cuQword *x, long *pr64,
			     exflib_uQword *mem)
{
    exflib_float_floor(ip, x, *pr64, mem);
    return;
}

void exflib_float_floor_f77__(exflib_uQword *ip,
			      exflib_cuQword *x, long *pr64,
			      exflib_uQword *mem)
{
    exflib_float_floor(ip, x, *pr64, mem);
    return;
}

void exflib_float_floor_f90_(exflib_uQword **ip,
			     exflib_cuQword **x, long *pr64,
			     exflib_uQword **mem)
{
    exflib_float_floor(*ip, *x, *pr64, *mem);
    return;
}

void exflib_float_floor_f90__(exflib_uQword **ip,
			      exflib_cuQword **x, long *pr64,
			      exflib_uQword **mem)
{
    exflib_float_floor(*ip, *x, *pr64, *mem);
    return;
}

/* End of file */
