/* -*- C -*- */
/* Time-stamp: "2006-04-03 16:00:02 fujiwara" */

/**********************************************************************/
/*  asin of exfloat type number                                       */
/* 	                                                              */
/*  Copyright (C) 2004, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id:$";

#include <stdio.h>
#include <stdlib.h>
#include "exfloat.h"
#include "func.h"

/**********************************************************************/
/*  y[] : asin(x), (1+pr64) elements                                  */
/*  x[] : input,   (1+pr64) elements                                  */
/*  w[] : work area, (1+pr64+guard)*5 elements                        */
/**********************************************************************/

exflib_ulong exflib_float_asin(exflib_uQword y[], exflib_cuQword x[],
			       const long pr64,
			       exflib_uQword w[])
{
    const long guard = 1;		/* gurad elements */
    const long pr64g = pr64 + guard;	/* extended precision */
    exflib_ulong ret;

    const long MAX_REDUCE = 100;
    long reduce, minus, minus2;

    exflib_uQword *xx    = w;
    exflib_uQword *asinx = w + (1+pr64g);
    exflib_uQword *term  = w + (1+pr64g)*2;
    exflib_uQword *xsq   = w + (1+pr64g)*3;
    exflib_uQword *tmp   = w + (1+pr64g)*4;
    exflib_uQword frac;

    long i, n;

    /* x == 0 or x == -0 */
    if((x[0] & (~EXFLIB_MASK_SGN)) == 0){
	y[0] = x[0];
	for(i = 1; i <= pr64; i++)
	    y[i] = 0;
	return 0;
    }

    /* x == 1 or x == -1 */
    frac = (x[0] & (~EXFLIB_MASK_SGN)) - EXFLIB_BIAS;
    for(i = 1; (!frac) && (i <= pr64); i++)
	    frac |= x[i];
    if(frac == 0){

	exflib_float_ctoef("pi", asinx, pr64+1);

	y[0] = asinx[0] - 1;		/* divided by 2 (asin(1) == pi/2) */
	if(x[0] & EXFLIB_MASK_SGN) 	/* x == -1 */
	    y[0] |= EXFLIB_MASK_SGN;

	for(i = 1; i <= pr64; i++)
	    y[i] = asinx[i];

	ret = (sizeof(long) == 8)? asinx[pr64+1]: (asinx[pr64+1] >> 32);
	return ret;
    }

    /* domain check */
    /* |x| > 1, error */
    if((x[0] & (~EXFLIB_MASK_SGN)) >= EXFLIB_BIAS){
	fprintf(stderr, "asin : domain error.\n");
	exit(1);
    }

    /* zero extension of argument x */
    for(i = 0; i <= pr64; i++)
	xx[i] = x[i];
    while(i <= pr64g)
	xx[i++] = 0;

    asinx[0] = term[0] = EXFLIB_BIAS;
    for(i = 1; i < pr64g; i++)
	asinx[i] = term[i] = 0;

    /* reduction of argument x */
    reduce = minus = minus2 = 0;

    if(xx[0] & EXFLIB_MASK_SGN){ 	/* x < 0 */
	minus = 1;
	xx[0] += EXFLIB_MASK_SGN;
    }

    term[0] = 0x3ffffffffffffffeULL;		/* term = 1/2 */
    while(exflib_float_lt(term, xx, pr64g) && (reduce < MAX_REDUCE)){
	reduce++;
	exflib_float_mul(xx, xx, xx, pr64g);
	xx[0] += 1;
	exflib_float_sub(xx, xx, asinx, pr64g);
    }

    term[0] = 0x3fffffffffffffffULL;		/* term = 1 */
    if(xx[0] & EXFLIB_MASK_SGN){	/* xx < 0 */
	minus2 = 1;
	xx[0] += EXFLIB_MASK_SGN;
    }

    exflib_float_mul(xsq, xx, xx, pr64g);

    /* Taylor expansion */
    /* In the future, change it to Horner's method */
    for(n = 2; ; n += 2){

	exflib_float_mul(term, term, xsq, pr64g);
	exflib_float_idiv(term, term, n, pr64g);
	exflib_float_imul(term, term, n-1, pr64g);

	exflib_float_idiv(tmp, term, n+1, pr64g);
	exflib_float_add(tmp, asinx, tmp, pr64g);

	/* comparing (1+pr64) elements */
	for(i = 0; (i <= 1 + pr64) && (tmp[i] == asinx[i]); i++)
	    continue;
	if(i >= 1 + pr64)
	    break;

	/* renewal */
        {
            exflib_uQword *swap = asinx;
            asinx = tmp;
            tmp = swap;
        }
    }
	    
    exflib_float_mul(asinx, xx, asinx, pr64g);

    /* reset if x has reduced */
    if(minus2)
	asinx[0] += EXFLIB_MASK_SGN;

    exflib_float_ctoef("pi", tmp, pr64g);
    tmp[0] -= 2; 	/* tmp = pi/4 */
    while(reduce-- > 0){
	asinx[0] -= 1;  /* asinx /= 2; */
	exflib_float_add(asinx, asinx, tmp, pr64g);
    }
    if(minus)
	asinx[0] += EXFLIB_MASK_SGN;

    for(i = 0; i <= pr64; i++)
	y[i] = asinx[i];
    ret = (sizeof(long) == 8)? asinx[pr64+1]: (asinx[pr64+1] >> 32);

    return ret;
}

/* End of func_asin.c */
