/* -*- C -*- */
/* Time-stamp: "2006-03-17 15:53:04 fujiwara" */

/**********************************************************************/
/*  atan of exfloat type number                                       */
/* 	                                                              */
/*  Copyright (C) 2004, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id:$";

#include <stdio.h>
#include <math.h>
#include "exfloat.h"

#include "func.h"

/**********************************************************************/
/*  argument reduction for atan                                       */
/**********************************************************************/
long exflib_atan_arg_reduction(exflib_uQword x[], long pr64)
{
    long k = 0, i;
    exflib_uQword c[1 + EXFLIB_CONST_MAX_PR];

    c[0] = EXFLIB_BIAS;
    for(i = 1; i <= pr64; i++)
	c[i] = 0;

    /* x <= 0 */
    if((x[0] & EXFLIB_SIGN) == EXFLIB_SIGN){
	k |= 1;
	x[0] ^= EXFLIB_SIGN;
    }

    /* x > 1 */
    if(x[0] >= EXFLIB_BIAS){
	k |= 2;
	exflib_float_div(x, c, x, pr64);
    }

    /* x > 1/2 */
    if(x[0] >= EXFLIB_BIAS-1){
	k |= 4;
	
	c[0] = EXFLIB_BIAS + 1;	/* c = 2 */
	exflib_float_add(x, x, c, pr64);

	c[0] = EXFLIB_BIAS + 2;	/* c = 5 */
	c[1] = 0x4000000000000000ULL;
	exflib_float_div(x, c, x, pr64);

	c[0] = EXFLIB_BIAS + 1;	/* c = 2 */
	c[1] = 0;
	exflib_float_sub(x, c, x, pr64);
    }

    /* x > 1/4 */
    if(x[0] >= EXFLIB_BIAS-2){
	k |= 8;

	c[0] = EXFLIB_BIAS + 2;	/* c = 4 */
	exflib_float_add(x, x, c, pr64);

	c[0] = EXFLIB_BIAS + 4;	/* c = 17 */
	c[1] = 0x1000000000000000ULL;
	exflib_float_div(x, c, x, pr64);

	c[0] = EXFLIB_BIAS + 2;	/* c = 4 */
	c[1] = 0;
	exflib_float_sub(x, c, x, pr64);
    }

    /* x > 1/8 */
    if(x[0] >= EXFLIB_BIAS-3){
	k |= 16;

	c[0] = EXFLIB_BIAS + 2;	/* c = 4 */
	exflib_float_add(x, x, c, pr64);

	c[0] = EXFLIB_BIAS + 4;	/* c = 17 */
	c[1] = 0x1000000000000000ULL;
	exflib_float_div(x, c, x, pr64);

	c[0] = EXFLIB_BIAS + 2;	/* c = 4 */
	c[1] = 0;
	exflib_float_sub(x, x, c, pr64);
    }

    return k;
}

/**********************************************************************/
/*  recovering return value for atan                                  */
/**********************************************************************/
void exflib_atan_recover(exflib_uQword x[], long k, long pr64)
{
    static exflib_uQword pi_h[] = {
#include "../const/pi.exfloat"	
    };
    static exflib_uQword atan_q[] = {
#include "../const/atan_q.exfloat"
    };
    static exflib_uQword atan_h[] = {
#include "../const/atan_h.exfloat"
    };

    pi_h[0] = EXFLIB_BIAS;

    if(k & 16)
	exflib_float_sub(x, atan_q, x, pr64);
    if(k & 8)
	exflib_float_add(x, atan_q, x, pr64);
    if(k & 4)
	exflib_float_add(x, atan_h, x, pr64);
    if(k & 2)
	exflib_float_sub(x, pi_h, x, pr64);
    if(k & 1)
	x[0] ^= EXFLIB_SIGN;

    return;
}

/**********************************************************************/
/*  v = atan(x)                                                       */
/**********************************************************************/
void exflib_atan_horner(exflib_uQword x[], exflib_uQword v[], exflib_uQword table[], long pr64)
{
    exflib_uQword xx[1+EXFLIB_CONST_MAX_PR];
    exflib_Qword ee;
    long e, i, n;

    ee = (x[0] & ~EXFLIB_SIGN) - EXFLIB_BIAS;
    if(ee <= -2.0 - 32*pr64){
	// return x
	for(i = 0; i <= pr64; i++)
	    v[i] = x[i];
	return;
    }

    e = (long)ee;
    n = exflib_taylor_tranc_atan(pr64, e);

    exflib_exfloat_to_array(x, e, v, pr64+1);
    exflib_array_mul(xx, v, v, pr64+1);

    for(i = 0; i <= pr64; i++)
	v[i] = 0;
    exflib_invadd(v, n+1, table, 1, pr64+1);

    while(n > 0){
	exflib_array_mul(v, v, xx, pr64+1);
	exflib_invadd(v, n, table, -1, pr64+1);
	n--;
    }

    if(v[0] == 1){
	v[0] = EXFLIB_BIAS;
	for(i = 1; i <= pr64; i++)
	    v[i] = 0;
    } else {
	exflib_sll_onebit(v, pr64 + 1);
	v[0] = EXFLIB_BIAS-1;
    }
    exflib_float_mul(v, v, x, pr64);

    return;
}

/**********************************************************************/
/*  y[] : atan(x), (1+pr64) elements                                  */
/*  x[] : input,   (1+pr64) elements                                  */
/*  w[] : work area, (1+pr64+guard)*2 elements                        */
/*  inv_table[] : inverse of 1,2,...,n                                */
/**********************************************************************/

exflib_ulong exflib_float_atan(exflib_uQword y[], exflib_cuQword x[],
			       const long pr64,
			       exflib_uQword w[], exflib_uQword *inv_table)
{
    const long guard = 1;		/* gurad elements */
    const long pr64g = pr64 + guard;	/* extended precision */
    exflib_uQword *xx = w;
    exflib_uQword  *r = w+(1+pr64g);
    exflib_ulong ret;
    
    long k, i;

    /* x == 0, then return zero */
    if((x[0] & (~EXFLIB_MASK_SGN)) == 0){
	y[0] = x[0];
	for(i = 1; i <= pr64; i++)
	    y[i] = 0;
	return 0;
    }

    /* extending input x[], zero extension */
    for(i = 0; i <= pr64; i++)
	xx[i] = x[i];
    while(i <= pr64g)
	xx[i++] = 0;

    k = exflib_atan_arg_reduction(xx, pr64g);
    exflib_atan_horner(xx, r, inv_table, pr64g);
    if(k)
	exflib_atan_recover(r, k, pr64g);

    /* result and return value */
    for(i = 0; i <= pr64; i++)
	y[i] = r[i];

    ret = (sizeof(long) == 8)? r[pr64+1]: (r[pr64+1] >> 32);
    return ret;
}

/* End of func_atan.c */
