/* -*- C -*- */
/* $Id$ */

/**********************************************************************/
/*  Copyright (C) 2005, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: $";

#include "exfloat.h"
#include "func.h"

/**********************************************************************/
/*  Tow's Complements for fixed-point format (array)                  */
/*    b[size] = 1 - a[size]                                           */
/**********************************************************************/

void exflib_array_2s_complement(exflib_cuQword a[], exflib_uQword b[], long size)
{
    long i;

    b[0] = 0;
    for(i = 1; i < size; i++)
	b[i] = ~a[i];
    for(i = size-1; (i >= 0) && b[i] == 0xffffffffffffffffULL; i--)
	continue;
    if(i >= 0)
	b[i] += 1;

    return;
}

/**********************************************************************/
/* Convert exfloat type floatp[size] to fixed-point type fixedp[size] */
/* Assumption : e <= 0                                                */
/**********************************************************************/

void exflib_exfloat_to_array(exflib_cuQword floatp[], long e,
			     exflib_uQword fixedp[], long size)
{
    long i, ii;
    long n, r;
    exflib_cuQword hidden_one = 1;

    if(e >= 0){
	for(i = 0; i < size; i++)
	    fixedp[i] = floatp[i];
	return;
    }

    if(e <= -(size-1)*64){
	for(i = 0; i < size; i++)
	    fixedp[i] = 0;
	if(e == (size-1)*64)	/* hidden one */
	    fixedp[size-1] = hidden_one;
	return;
    }

    n = (-e)/64;
    r = (-e)%64;

    if(r){
	for(i = 0; i <= n; i++)
	    fixedp[i] = 0;

	fixedp[i++] = (hidden_one << (64-r)) | (floatp[1] >> r);

	for(ii = 1; i < size; i++, ii++)
	    fixedp[i] = (floatp[ii] << (64-r)) | (floatp[ii+1] >> r);
    } else {

	for(i = 0; i < n; i++)
	    fixedp[i] = 0;
	fixedp[i++] = hidden_one;
	for(ii = 1; i < size; i++, ii++)
	    fixedp[i] = floatp[ii];
    }
	    
    return;
}

/**********************************************************************/
/* Convert fixed-point type number v to floating-point type format    */
/* Assumption : v[0] >= 1                                             */
/**********************************************************************/

void exflib_array_to_exfloat(exflib_uQword v[], long pr64)
{
    exflib_Qword t = v[0];
    long s = 0;

    /* amount of shift */
    while(t != 1){
	t >>= 1;
	s++;
    }

    if(s > 0)
	while(pr64 >= 1){
	    v[pr64] = (v[pr64-1] << (64-s)) | (v[pr64] >> s);
	    pr64--;
	}

    v[pr64] = EXFLIB_BIAS + s;

    return;
}

/**********************************************************************/
/*  Shift left logical by one bit                                     */
/**********************************************************************/

void exflib_sll_onebit(exflib_uQword v[], long size)
{
    long i;
    
    for(i = 0; i < size-1; i++)
	v[i] = (v[i] << 1) | (v[i+1] >> 63);

    v[size-1] = v[size-1] << 1;
    
    return;
}

/* End of func_common.c */
