/* -*- C -*- */
/* Time-stamp: "2006-02-27 22:38:09 fujiwara" */

/**********************************************************************/
/*  Copyright (C) 2005, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: $";

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "exfloat.h"
#include "func.h"

/**********************************************************************/
/*  v = exp(x)                                                        */
/*   0 <= x < ln(2)                                                   */
/**********************************************************************/

void exflib_array_exp(exflib_uQword x[], exflib_uQword v[], long pr64)
{
    exflib_uQword xx[1+EXFLIB_CONST_MAX_PR];
    exflib_Qword ee;
    long i, e, n, dummy;

    ee = (x[0] & ~EXFLIB_SIGN) - EXFLIB_BIAS;
    if(ee < -64*pr64 - 2){
	v[0] = EXFLIB_BIAS;
	for(i = 1; i <= pr64; i++)
	    v[i] = 0;
	return;
    }

    e = (long) ee;
    n = exflib_taylor_tranc_exp(pr64, e);
    exflib_exfloat_to_array(x, e, xx, pr64+1);

    v[0] = 1;
    for(i = 1; i <= pr64; i++)
	v[i] = 0;

    while(n > 0){
	dummy = 0;
	exflib_array_idiv(v, v, n, pr64+1, &dummy);
	exflib_array_mul(v, v, xx, pr64+1);
	v[0] += 1;
	n--;
    }

    /* v = exp(x) \in [1, 2] */
    if(v[0] == 1){
	v[0] = EXFLIB_BIAS;
    } else {
	v[0] = EXFLIB_BIAS + 1;
	for(i = 1; i <= pr64; i++)
	    v[0] = 0;
    }

    return;
}

/***********************************************************************/
/*  y[]   : exp(x),  	(1+pr64) elements                              */
/*  x[]   : input,    	(1+pr64) elements                              */
/*                                                                     */
/*  w[] : work area, 	(1+pr64+guard)*2 elements                      */
/***********************************************************************/

exflib_ulong exflib_float_exp(exflib_uQword y[], exflib_cuQword x[], const long pr64, exflib_uQword w[])
{
    const long guard = 1;		/* guard element */
    const long pr64g = pr64 + guard;	/* extended precision */

    exflib_uQword *xx = w;
    exflib_uQword *r =  w + (1+pr64g);
    exflib_ulong ret;
    
    long sign = 1;
    exflib_Qword k;
    long i;

    /* zero check */
    if((x[0] == EXFLIB_MASK_SGN) || (x[0] == 0)){
	y[0] = EXFLIB_BIAS;
	for(i = 1; i <= pr64; i++)
	    y[i] = 0;
	return 0;
    }

    /* zero extension */
    for(i = 0; i <= pr64; i++)
	xx[i] = x[i];
    for(; i <= pr64g; i++)
	xx[i] = 0;

    /* negate */
    if((xx[0] & EXFLIB_MASK_SGN) == EXFLIB_MASK_SGN){
	sign = -1;
	xx[0] ^= EXFLIB_MASK_SGN;
    }

    /* additive reduction */
    exflib_add_reduction(xx, 2, r, pr64g, &k);
    exflib_array_exp(r, xx, pr64g);

    if(sign == 1){

	for(i = 0; i <= pr64; i++)
	    y[i] = xx[i];
	y[0] += k;

	if(sizeof(long) == 8)	/* 64-bit architecture */
	    ret = xx[pr64+1];
	else if(sizeof(long) == 4)	/* 32-bit architecture */
	    ret = xx[pr64+1] >> 32;

    } else {

	r[0] = EXFLIB_BIAS;
	for(i = 1; i <= pr64g; i++)
	    r[i] = 0;
	ret = exflib_float_div(y, r, xx, pr64);
	y[0] -= k;
    }

    return ret;
}

/* End of func_exp.c */
