/* -*- C -*- */
/* Time-stamp: "2006-03-17 00:36:35 fujiwara" */

/**********************************************************************/
/*  Copyright (C) 2006, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: $";
#include "exfloat.h"

/**********************************************************************/
/*  modf, place integer part of x in *ip, fractional part of x in fp  */
/**********************************************************************/

void exflib_float_modf(exflib_cuQword x[],
		       exflib_uQword ip[], exflib_uQword fp[], long pr64)
{
    exflib_Qword e = (x[0] & ~EXFLIB_MASK_SGN) - EXFLIB_BIAS;
    exflib_uQword mask = 0xffffffffffffffffULL;
    int i;

    /* e < 0, i.e., |x| < 1 */
    if(e < 0){
	for(i = 0; i <= pr64; i++){
	    fp[i] = x[i];
	    ip[i] = 0;
	}
	return;
    }

    /* In the following, e >= 0, i.e., |x| >= 1 */
    ip[0] = x[0];

    i = 1;
    for(; e-64 >= 0; e -= 64, i++)
	ip[i] = x[i];

    if(e) {
	ip[i] = x[i] & (mask << (64 - e));
	i++;
    }

    while(i <= pr64)
	ip[i++] = 0;

    exflib_float_sub(fp, x, ip, pr64);

    return;
}

/**********************************************************************/
/*  ceil, smallest integer not less than x                            */
/*                                                                    */
/*  mem[] has (1+pr64) elements                                       */
/**********************************************************************/

void exflib_float_ceil(exflib_uQword ip[], exflib_cuQword x[], long pr64,
		       exflib_uQword mem[])
{
    int i;

    if(x[0] == 0){
	for(i = 0; i <= pr64; i++)
	    ip[i] = 0;
	return;
    }

    exflib_float_modf(x, ip, mem, pr64);
    
    if((x[0] & EXFLIB_MASK_SGN) || (mem[0] == 0))
	return;

    /* Let mem = 1 */
    mem[0] = 0x3fffffffffffffffULL;
    for(i = 1; i <= pr64; i++)
	mem[i] = 0;

    exflib_float_add(ip, ip, mem, pr64);

    return;
}

/**********************************************************************/
/*  floor, largest integer not greater than x                         */
/*                                                                    */
/*  mem[] has (1+pr64) elements                                       */
/**********************************************************************/

void exflib_float_floor(exflib_uQword ip[], exflib_cuQword x[], long pr64,
			exflib_uQword mem[])
{
    int i;

    if(x[0] == 0){
	for(i = 0; i <= pr64; i++)
	    ip[i] = 0;
	return;
    }

    exflib_float_modf(x, ip, mem, pr64);

    if(!(x[0] & EXFLIB_MASK_SGN) || (mem[0] == 0))
	return;

    /* Let mem = 1 */
    mem[0] = 0x3fffffffffffffffULL;
    for(i = 1; i <= pr64; i++)
	mem[i] = 0;

    exflib_float_sub(ip, ip, mem, pr64);

    return;
}


/* End of file */
