/* -*- C -*- */
/* Time-stamp: "2006-03-17 14:21:32 fujiwara" */

/**********************************************************************/
/*  power of exfloat type number                                      */
/* 	                                                              */
/*  Copyright (C) 2004, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id:$";

#include <stdio.h>
#include <stdlib.h>
#include "exfloat.h"

#include "func.h"

/**********************************************************************/
/*  p[]      : pow(x,y), (1+pr64) elements                            */
/*  x[], y[] : input,   (1+pr64) elements                             */
/*  w[] : work area, (1+pr64+guard)*2 elements                        */
/*  table[]  : inverse of 1, 2, 3, ..., n (for internal log call)     */
/**********************************************************************/

exflib_ulong exflib_float_pow(exflib_uQword p[],
			      exflib_cuQword x[], exflib_cuQword y[],
			      const long pr64, exflib_uQword w[],
			      exflib_uQword table[])
{
    const long guard = 1;		/* gurad elements */
    const long pr64g = pr64 + guard;	/* extended precision */
    exflib_uQword *ip = w;
    exflib_uQword *fp = w+(1+pr64g);
    exflib_ulong ret;
    
    long even = 1;
    long i;

    /* domain check */
    /* if (x == 0 and y <= 0), domain error */
    if((x[0] & (~EXFLIB_MASK_SGN)) == 0){
	if((y[0] == 0) || (y[0] & EXFLIB_MASK_SGN)){
	    fprintf(stderr, "pow : domain error, base is zero.\n");
	    exit(1);
	}
    }

    if(x[0] & EXFLIB_MASK_SGN){		/* x <= 0 */

	/* y is not an integer, error */
	exflib_float_modf(y, ip, fp, pr64);
	if((fp[0] & (~EXFLIB_MASK_SGN)) == 0){
	    fprintf(stderr, "pow : domain error, base is negative.\n");
	    exit(1);
	}

	/* integer part of y is odd or even ? */
	exflib_float_idiv(fp, ip, 2, pr64);
	exflib_float_modf(fp, ip, p, pr64);
	if(p[0] & (~EXFLIB_MASK_SGN))	/* y is odd */
	    even = 1;

	/* negate x */
	p[0] = x[0] & (~EXFLIB_MASK_SGN);
	for(i = 1; i <= pr64; i++)
	    p[i] = x[i];

	exflib_float_log(p, p, pr64, w, table);
	exflib_float_mul(p, p, y, pr64);
	ret = exflib_float_exp(p, p, pr64, w);

    } else {

	/* x >= 0 */
	exflib_float_log(p, x, pr64, w, table);
	exflib_float_mul(p, p, y, pr64);
	ret = exflib_float_exp(p, p, pr64, w);
    }

    return ret;
}

/* End of func_pow.c */
