/* -*- C -*- */
/* Time-stamp: "2006-03-17 14:57:00 fujiwara" */

/**********************************************************************/
/*  Copyright (C) 2005, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: $";

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "exfloat.h"
#include "func.h"

/***********************************************************************/
/*  z[]   : pow(x,y),  	(1+pr64) elements                              */
/*  x[]   : input,    	(1+pr64) elements                              */
/*  y     : input                                                      */
/*                                                                     */
/*  w[] : work area, 	(1+pr64+guard)*2 elements                      */
/***********************************************************************/

exflib_ulong exflib_float_pow_long(exflib_uQword z[], exflib_cuQword x[], long y, const long pr64, exflib_uQword w[])
{
    const long guard = 1; // guard digit
    const long pr64g = pr64 + guard;
    exflib_uQword *powx = w;
    exflib_uQword *xsq = w + (1+pr64g);

    exflib_ulong ret;
    long i;

    /* domain check */
    /* if base == 0 and e <= 0, then domain error */
    if((x[0] == 0) || (x[0] == EXFLIB_MASK_SGN)){
	if(y <= 0){
	    fprintf(stderr, "pow : domain error\n");
	    exit(1);
	}
	return 0;
    }

    /* initialize */
    powx[0] = EXFLIB_BIAS;
    for(i = 1; i <= pr64g; i++)
	powx[i] = 0;

    for(i = 0; i <= pr64; i++)
	xsq[i] = x[i];
    for(; i <= pr64g; i++)
	xsq[i] = 0;

    if(y < 0){
	exflib_float_div(xsq, powx, xsq, pr64g);
	y = -y;
    }

    for(; y > 0; y >>= 1){

	if(y % 2)
	    exflib_float_mul(powx, powx, xsq, pr64g);
	exflib_float_mul(xsq, xsq, xsq, pr64g);
    }

    for(i = 0; i <= pr64; i++)
	z[i] = powx[i];

    if(sizeof(long) == 8)	/* 64-bit architecture */
	ret = powx[pr64+1];
    else if(sizeof(long) == 4)	/* 32-bit architecture */
	ret = powx[pr64+1] >> 32;

    return ret;
}
    
/* End of func_pow_long.c */
