/* -*- C -*- */
/* Time-stamp: "2006-02-16 16:00:27 fujiwara" */

/**********************************************************************/
/*  Copyright (C) 2005, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: $";

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "exfloat.h"
#include "func.h"

/**********************************************************************/
/*  v = sin(x)                                                        */
/*   -pi/4 <= x <= pi/4                                               */
/**********************************************************************/

void exflib_array_sin(exflib_uQword x[], exflib_uQword v[], long pr64)
{
    exflib_uQword xx[1+EXFLIB_CONST_MAX_PR];
    exflib_Qword ee;
    long e, i, dummy = 0;
    long n;

    ee = (x[0] & ~EXFLIB_MASK_SGN) - EXFLIB_BIAS;
    if(ee <= -32*pr64 - 1){
	for(i = 0; i <= pr64; i++)
	    v[i] = x[i];
	return;
    }

    e = (long)ee;
    n = exflib_taylor_tranc_sin(pr64, e);
    exflib_exfloat_to_array(x, e, v, pr64+1);
    exflib_array_mul(xx, v, v, pr64+1);
    for(i = 0; i <= pr64; i++)
	v[i] = xx[i];

    while(n > 0){
	dummy = 0;
	exflib_array_idiv(v, v, 2*n*(2*n+1), pr64+1, &dummy);
	exflib_array_mul(v, v, xx, pr64+1);
	exflib_array_2s_complement(v, v, pr64+1);
	n--;
    }

    /* v = sin(x)/x \in [0.900316..., 1] */
    if(v[0] == 0){
	exflib_sll_onebit(v, pr64+1);
	v[0] = EXFLIB_BIAS-1;
    }else{
	v[0] = EXFLIB_BIAS;
    }

    exflib_float_mul(v, x, v, pr64);

    return;
};


/***********************************************************************/
/*  y[]   : sin(x),  	(1+pr64) elements                              */
/*  x[]   : input,    	(1+pr64) elements                              */
/*                                                                     */
/*  w[] : work area, 	(1+pr64+guard)*2 elements                      */
/***********************************************************************/

exflib_ulong exflib_float_sin(exflib_uQword y[], exflib_cuQword x[], const long pr64, exflib_uQword w[])
{
    const long guard = 1;		/* guard element */
    const long pr64g = pr64 + guard;	/* extended precision */
    exflib_uQword *xx = w;
    exflib_uQword *r = w + (1+pr64g);
    exflib_ulong ret;

    long sign = 1, k;

    long i;
    exflib_Qword dummy;
    
    /* zero check */
    if((x[0] == EXFLIB_MASK_SGN) || (x[0] == 0)){
	y[0] = x[0];
	return 0;
    }

    /* zero extension */
    for(i = 0; i <= pr64; i++)
	xx[i] = x[i];
    for(; i <= pr64g; i++)
	xx[i] = 0;

    /* sign check */
    if(xx[0] & EXFLIB_MASK_SGN){
	sign = -1;
	xx[0] ^= EXFLIB_MASK_SGN;
    }

    // additive reduction
    k = exflib_add_reduction(xx, 3, r, pr64g, &dummy);

    switch(k){
      case 0:
        exflib_array_sin(r, xx, pr64g);
	break;
      case 1:
        exflib_array_cos(r, xx, pr64g);
	break;
      case 2:
        exflib_array_sin(r, xx, pr64g);
        sign *= -1;
	break;
      case 3:
        exflib_array_cos(r, xx, pr64g);
        sign *= -1;
	break;
    }

    /* negate */
    if(sign == -1)
	xx[0] ^= EXFLIB_MASK_SGN;

    for(i = 0; i <= pr64; i++)
	y[i] = xx[i];

    if(sizeof(long) == 8)	/* 64-bit architecture */
	ret = xx[pr64+1];
    else if(sizeof(long) == 4)	/* 32-bit architecture */
	ret = xx[pr64+1] >> 32;

    return ret;
}

/* End of func_sin.c */
