/* -*- C -*- */
/* Time-stamp: "2005-02-02 17:27:41 fujiwara" */

/**********************************************************************/
/*  Copyright (C) 2002, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/
static char rcsid[] = "$Id:$";

#include <math.h>
#include "exfloat.h"
#include "func.h"

const long resi = 0;

/**********************************************************************/
/*  sine function                                                     */
/**********************************************************************/

long exflib_taylor_tranc_sin_calc(long arrays, long e)
{
    double c = (e == -1)? -log(atan(1.0))/log(2.0): -e-1;
    long n = 0;

    double sum = 2*c
	+ log(1.0)/log(2.0) + log(2.0)/log(2.0) + log(3.0)/log(2.0);

    while(sum < 64*arrays + 1 + resi){

	n += 1;
	sum += 2*c
	    + log(2.0*n+2.0)/log(2.0) + log(2.0*n+3.0)/log(2.0);
    }
    return n;
}

long exflib_taylor_tranc_sin(long arrays, long e)
{
    static const long tbl[EXFLIB_FUNCARG_MAX_PR64][-EXFLIB_FUNCARG_EXP_LEAST]	= {
#include "../trancation/sin.tbl"
    };

    if(e <= -32*arrays - 1)
	return 0;
    if(arrays >= EXFLIB_FUNCARG_MAX_PR64)
	return exflib_taylor_tranc_sin_calc(arrays, e);
    if(e < EXFLIB_FUNCARG_EXP_LEAST) e = EXFLIB_FUNCARG_EXP_LEAST;
    return tbl[arrays-1][-e-1];
}

/**********************************************************************/
/*  cosine function                                                   */
/**********************************************************************/

long exflib_taylor_tranc_cos_calc(long arrays, long e)
{
    double c = (e == -1)? -log(atan(1.0))/log(2.0): -e-1;
    long n = 0;

    double sum = 2*c
	+ log(1.0)/log(2.0) + log(2.0)/log(2.0);

    while(sum < 64*arrays + 1 + resi){

	n += 1;
	sum += 2*c
	    + log(2.0*n+1.0)/log(2.0) + log(2.0*n+2.0)/log(2.0);
    }
    return n;
}

long exflib_taylor_tranc_cos(long arrays, long e)
{
    static const long tbl[EXFLIB_FUNCARG_MAX_PR64][-EXFLIB_FUNCARG_EXP_LEAST] = {
#include "../trancation/cos.tbl"
    };

    if(e <= -32*arrays - 1)
	return 0;
    if(arrays >= EXFLIB_FUNCARG_MAX_PR64)
	return exflib_taylor_tranc_cos_calc(arrays, e);
    if(e < EXFLIB_FUNCARG_EXP_LEAST) e = EXFLIB_FUNCARG_EXP_LEAST;
    return tbl[arrays-1][-e-1];
}

/**********************************************************************/
/*  exponential function                                              */
/**********************************************************************/

long exflib_taylor_tranc_exp_calc(long arrays, long e)
{
    double c = (e == -1)? log(1.0/log(2.0))/log(2.0): -e-1;
    long n = 0;
    
    double sum = c
	+ log(1.0)/log(2.0);

    while(sum < 64*arrays + resi + 1){

	n += 1;
	sum += c
	    + log((double)n)/log(2.0);
    }

    return n;
}

long exflib_taylor_tranc_exp(long arrays, long e)
{
    static const long tbl[EXFLIB_FUNCARG_MAX_PR64][-EXFLIB_FUNCARG_EXP_LEAST] = {
#include "../trancation/exp.tbl"
    };

    if(e < -64*arrays - 2)
	return 0;
    if(arrays >= EXFLIB_FUNCARG_MAX_PR64)
	return exflib_taylor_tranc_exp_calc(arrays, e);
    if(e < EXFLIB_FUNCARG_EXP_LEAST) e = EXFLIB_FUNCARG_EXP_LEAST;
    return tbl[arrays-1][-e-1];
}

/**********************************************************************/
/*  natural logarithmic function                                      */
/**********************************************************************/

long exflib_taylor_tranc_log_calc(long arrays, long e)
{
    long n = 0;
    double val = -2*(1+e) + 3*log(3.0)/log(2.0) - 2.0;

    if(e == -1) return 0;

    while(val < 64*arrays + resi + 1){

	n += 1;
	val = -(2*n+2)*(1+e) + log(9.0*(2.0*n+3.0)) - 2.0;
    }

    return n;
}

long exflib_taylor_tranc_log(long arrays, long e)
{
    static const long tbl[EXFLIB_FUNCARG_MAX_PR64][-EXFLIB_FUNCARG_EXP_LEAST] = {
#include "../trancation/log.tbl"
    };

    if(e < 1.5*log(3.0)/log(2.0) - 1.5 - 32*arrays)
	return 0;
    if(arrays >= EXFLIB_FUNCARG_MAX_PR64)
	return exflib_taylor_tranc_log_calc(arrays, e);
    if(e < EXFLIB_FUNCARG_EXP_LEAST) e = EXFLIB_FUNCARG_EXP_LEAST;
    return tbl[arrays-1][-e-1];
}

/**********************************************************************/
/*  arc tangent function                                              */
/**********************************************************************/

long exflib_taylor_tranc_atan_calc(long arrays, long e)
{
    long n = 0;
    double val = -(e+1)*(2*n+2) + log(2.0*n+3.0)/log(2.0);

    if(e >= -2) return 0;

    while(val < 64*arrays + resi){

	n += 1;
	val = -(e+1)*(2*n+2) + log(2.0*n+3.0)/log(2.0);
    }

    return n;
}

long exflib_taylor_tranc_atan(long arrays, long e)
{
    static const long tbl[EXFLIB_FUNCARG_MAX_PR64][-EXFLIB_FUNCARG_EXP_LEAST] = {
#include "../trancation/atan.tbl"
    };

    if(e <= -2 - 32*arrays)
	return 0;
    if(arrays >= EXFLIB_FUNCARG_MAX_PR64)
	return exflib_taylor_tranc_atan_calc(arrays, e);
    if(e < EXFLIB_FUNCARG_EXP_LEAST) e = EXFLIB_FUNCARG_EXP_LEAST;
    return tbl[arrays-1][-e-1];
}

/* End of taylor_tranc.c */
