// -*- C++ -*-
// Time-stamp: "2006-03-17 16:18:55 fujiwara"
// $Id$

//----------------------------------------------------------------------
//  Built-In Functions by Continued Fractions
//
//  Copyright (C) 2003, FUJIWARA Hiroshi
//        <fujiwara@acs.i.kyoto-u.ac.jp>
//----------------------------------------------------------------------
static const char rcsid[] = "$Id:$";

#define PRECISION 800
#include "exfloat.h"
#include <iostream>

using namespace std;

const long MAX_N = 10000;

exfloat xx;

#if 1
//----------------------------------------------------------------------
//  exp(x)
//----------------------------------------------------------------------
const char id[] = "contrfac exp";

exfloat func(exfloat &x){ return exp(x); }

exfloat upper = "0.8";

exfloat a(long n, exfloat &x)
{
    if(n%2 == 1) return x;
    return -x;
}

exfloat b(long n)
{
    if(n == 0) return 1;
    if(n%2 == 0) return 2;
    return n;
}

#elif 0
//----------------------------------------------------------------------
//  exp(x) -- other expansion
//----------------------------------------------------------------------
const char id[] = "contrfac exp2";

exfloat func(exfloat &x){ return exp(x); }

exfloat upper = "0.8";

exfloat a(long n, exfloat &x)
{
    static exfloat x_square = x*x;
    if(n == 1){
	x_square = x*x;
	return 2*x;
    }
    return x_square;
}

exfloat b(long n)
{
    if(n == 0) return 1;
    if(n == 1) return 2-xx;
    return 2*(2*n-1);
}

#elif 0
//----------------------------------------------------------------------
//  log(x)
//----------------------------------------------------------------------
const char id[] = "contrfac log";

exfloat func(exfloat &x){ return log(x); }

exfloat upper = 2;

exfloat a(long n, exfloat &x)
{
    exfloat xxx = x - 1;

    if(n == 1) return xxx;
    n /= 2;
    return n*xxx;
}

exfloat b(long n)
{
    if(n == 0) return 0;
    if(n%2 == 1) return n;
    return 2;
}


#elif 0				// 
//----------------------------------------------------------------------
//  tan(x)
//----------------------------------------------------------------------
const char id[] = "contrfac tan";

exfloat func(exfloat &x){ return tan(x); }

exfloat upper = "1.2";

exfloat a(long n, exfloat &x)
{
    static exfloat x_square = -x*x;
    if(n == 1){
	x_square = -x*x;
	return x;
    }
    return x_square;
}

exfloat b(long n)
{
    if(n == 0) return 0;
    return 2*n - 1;
}

#else

//----------------------------------------------------------------------
// atan(x)
//----------------------------------------------------------------------
const char id[] = "contrfac atan";

exfloat func(exfloat &x) { return atan(x); }

exfloat upper = "0.2";

exfloat a(long n, exfloat &x)
{
    static exfloat x_square = x*x;
    if(n == 1){
	x_square = x*x;
	return x;
    }
    n--;
    return (n*n)*x_square;
}

exfloat b(long n)
{
    if(n == 0) return 0;
    return 2*n - 1;
}

#endif

//----------------------------------------------------------------------
//  main function
//----------------------------------------------------------------------

int main()
{
    exfloat p[3], q[3], f[2];
    long n = 0;
    exfloat x;
    exfloat h = upper/1000;

    cout << "# " << id << endl;

    for(x = h; x <= upper; x += h){

	xx = x;

	p[0] = b(0);	p[1] = b(0)*b(1) + a(1, x);
	q[0] = 1;    	q[1] = b(1);

	f[0] = p[1]/q[1];

	for(n = 2; n <= MAX_N; n++){
	    p[2] = b(n)*p[1] + a(n, x)*p[0];
	    q[2] = b(n)*q[1] + a(n, x)*q[0];

	    f[1] = p[2]/q[2];

	    if(f[1] == f[0])
		break;

	    p[0] = p[1]; p[1] = p[2];
	    q[0] = q[1]; q[1] = q[2];
	    f[0] = f[1];
	}

	cout << setiosflags(ios::fixed) << x << '\t' << n << '\t'
	     << "# " << setiosflags(ios::scientific)
	     << f[1] << '-' << func(x) << '=' << f[1]-func(x) << endl;
	
    }

    return 0;
}

//----------------------------------------------------------------------
// Local Variables:
// compile-command: "g++ -Wall -O2"
// End:
//----------------------------------------------------------------------
