// -*- C++ -*-
// $Id$

//----------------------------------------------------------------------
//  Copyright (C) 2002, FUJIWARA Hiroshi
//        <fujiwara@acs.i.kyoto-u.ac.jp>
//----------------------------------------------------------------------
static const char rcsid[] = "$Id:$";

#define PRECISION 800
#include "exfloat.h"
#include <iostream>

using namespace std;

const long N_MAX = 100000;

#if 1
//----------------------------------------------------------------------
// exp(x)
//----------------------------------------------------------------------
const char id[] = "taylor exp";

exfloat func(exfloat &x) { return exp(x); }

exfloat upper = log(static_cast<exfloat>(2));

exfloat term(long n, exfloat &x)
{
    static exfloat xx = 1;

    if(n == 0){
	xx = 1;
	return 1;
    }
    xx *= x/n;
    return xx;
}

#elif 1
//----------------------------------------------------------------------
// log(x)
//----------------------------------------------------------------------
const char id[] = "taylor log";

exfloat func(exfloat &x) { return log(x); }

exfloat upper = 2;

exfloat term(long n, exfloat &x)
{
    static exfloat y = (x-1)/(x+1);
    static exfloat xx = 2*y;

    if(n == 0){
	y = (x-1)/(x+1);
	xx = 2*y;
	return xx;
    }

    xx *= y*y;
    return xx/(2*n+1);
}

#elif 0
//----------------------------------------------------------------------
// sin(x)
//----------------------------------------------------------------------
const char id[] = "taylor sin";

exfloat func(exfloat &x) { return sin(x); }

exfloat upper = "#PI/4";

exfloat term(long n, exfloat &x)
{
    static exfloat xx = x;

    if(n == 0){
	xx = x;
	return xx;
    }

    xx *= -x*x;
    xx /= (2*n)*(2*n+1);

    return xx;
}

#elif 0
//----------------------------------------------------------------------
// cos(x)
//----------------------------------------------------------------------
const char id[] = "taylor cos";

exfloat func(exfloat &x) { return cos(x); }

exfloat upper = "#PI/4";

exfloat term(long n, exfloat &x)
{
    static exfloat xx = 1;

    if(n == 0){
	xx = 1;
	return xx;
    }

    xx *= -x*x;
    xx /= (2*n-1)*(2*n);

    return xx;
}

#else
//----------------------------------------------------------------------
// atan(x)
//----------------------------------------------------------------------
const char id[] = "taylor atan";

exfloat func(exfloat &x) { return atan(x); }

exfloat upper = "1/8";

exfloat term(long n, exfloat &x)
{
    static exfloat xx = x;
    if(n == 0){
	xx = x;
	return xx;
    }
    xx *= -x*x;
    return xx/(2*n+1);
}

#endif

//----------------------------------------------------------------------
// main function
//----------------------------------------------------------------------

int main()
{
    exfloat value = 0, v = 0;
    exfloat x;
    long n;
    exfloat h = upper/100;

    cout << "# " << id << endl;

    for(x = h; x <= upper; x += h){

	value = v = 0;
	for(n = 0; n <= N_MAX; n++){

	    value = v + term(n, x);

	    if(v == value)
		break;

	    v = value;
	}

	cout << setiosflags(ios::fixed) << x << '\t' << n << '\t'
	     << "# " << setiosflags(ios::scientific)
	     << v << '-' << func(x) << '=' << v - func(x) << endl;
    }

    return 0;
}

//----------------------------------------------------------------------
// Local Variables:
// compile-command: "g++ -Wall -O2"
// End:
//----------------------------------------------------------------------
